<?php

class Celsus_Set_Operation_IntersectionTest extends PHPUnit_Framework_TestCase {

	/**
	 * The intersection object we test with.
	 *
	 * @var Celsus_Set_Operation_Intersection
	 */
	private $_intersection = null;

	public function setUp() {
		$this->_intersection = new Celsus_Set_Operation_Intersection('Celsus_Test_Set_Interface');
	}

	/**
	 * @expectedException Celsus_Exception
	 */
	public function testUndefinedMethodOnSetObjectThrowsException() {
		return $this->_intersection->unavailable_method();
	}

	/**
	 * @expectedException Celsus_Exception
	 *
	 * Self-shunted - this class does not implement the required interface.
	 */
	public function testCannotAddElementThatDoesntMeetInterfaceRequirements() {
		$this->_intersection->addElement($this);
	}

	public function testNoElementsReturnsFalse() {
		$this->assertFalse($this->_intersection->acceptable());
	}

	public function testIntersectionOfTwoPassesReturnsTrue() {
		$this->_intersection->addElement(new Celsus_Test_Set_Acceptable());
		$this->_intersection->addElement(new Celsus_Test_Set_Acceptable());
		$this->assertTrue($this->_intersection->acceptable());
	}

	public function testIntersectionOfPassAndFailReturnsFalse() {
		$this->_intersection->addElement(new Celsus_Test_Set_Acceptable());
		$this->_intersection->addElement(new Celsus_Test_Set_Unacceptable());
		$this->assertFalse($this->_intersection->acceptable());
	}

	public function testIntersectionOfTwoFailsReturnsFalse() {
		$this->_intersection->addElement(new Celsus_Test_Set_Unacceptable());
		$this->_intersection->addElement(new Celsus_Test_Set_Unacceptable());
		$this->assertFalse($this->_intersection->acceptable());
	}

}

//$today = date('Y-m-d');
//$tomorrow = date('Y-m-d', strtotime('tomorrow'));
//$july = date('Y-m-d', strtotime('2008-07-14'));
//$third_last_monday = new Celsus_Temporal_Expression_DayInMonth(1, -3);
//$second_friday = new Celsus_Temporal_Expression_DayInMonth(5, 2);
//$second_monday = new Celsus_Temporal_Expression_DayInMonth(1, 2);
//$every_two_months = new Celsus_Temporal_Expression_MonthsFromStart(date('Y-m-d', strtotime('2008-08-11')), 2);
//$tenth_of_the_month = new Celsus_Temporal_Expression_DayOfMonth(10);
//$last_day_of_the_month = new Celsus_Temporal_Expression_DayOfMonth(-1);
//
//$intersection = new Celsus_Set_Operation_Intersection('Celsus_Temporal_Expression_Interface');
//$intersection->addElements(array($every_two_months, $last_day_of_the_month));
//
//var_dump($intersection->includes(date('Y-m-d', strtotime('2008-08-08'))));
//var_dump($intersection->includes(date('Y-m-d', strtotime('2008-08-15'))));
//var_dump($intersection->includes(date('Y-m-d', strtotime('2008-09-12'))));
//var_dump($intersection->includes(date('Y-m-d', strtotime('2008-09-30'))));
//var_dump($intersection->includes(date('Y-m-d', strtotime('2008-10-10'))));
//var_dump($intersection->includes(date('Y-m-d', strtotime('2008-10-31'))));
//
//
////$union = new Celsus_Set_Operation_Union('Celsus_Temporal_Expression_Interface');
////$union->addElements(array($third_last_monday, $second_friday));
//
////$difference = new Celsus_Set_Operation_Difference('Celsus_Temporal_Expression_Interface');
////$difference->addInclude($union);
////$difference->addExclude($second_monday);
////
////var_dump($difference->includes($today));
////var_dump($difference->includes($tomorrow));
////var_dump($difference->includes(date('Y-m-d', strtotime('2002-07-15'))));
//
////var_dump($union->includes($today));
////var_dump($union->includes($tomorrow));
////var_dump($union->includes(date('Y-m-d', strtotime('2008-08-08'))));
//
//
////var_dump($element->includes($today));
////var_dump($element->includes($tomorrow));
////var_dump($element->includes($july));
////var_dump($element->includes(date('Y-m-d', strtotime('2002-08-12'))));



?>